import os
import sys
import argparse
import pytracking.evaluation.vot2020 as vot
import importlib
import os
import cv2
import numpy as np
import time
import torch
import seaborn as sns
import matplotlib.pyplot as plt

import sys
import random

sys.path.append('../')
import pytracking.evaluation.vot2020 as vot
from ltr.data.bounding_box_utils import masks_to_bboxes

def get_parameters(tracker_name, param_file_name):
    param_module = importlib.import_module('pytracking.parameter.{}.{}'.format(tracker_name, param_file_name))
    params = param_module.parameters()
    return params

def read_image(image_file: str):
        return cv2.cvtColor(cv2.imread(image_file), cv2.COLOR_BGR2RGB)


def _convert_anno_to_list(vot_anno):
    vot_anno = [vot_anno[0], vot_anno[1], vot_anno[2], vot_anno[3]]
    return vot_anno


def _convert_image_path(image_path):
    return image_path

def set_seed(seed):
    random.seed(seed)
    os.environ['PYTHONHASHSEED'] = str(seed)
    np.random.seed(seed)
    torch.manual_seed(seed)
    torch.cuda.manual_seed(seed)
    torch.backends.cudnn.benchmark = True
    torch.backends.cudnn.deterministic = True
    torch.backends.cudnn.enabled = True


def run_vot2020():
    seed =1000
    set_seed(seed)

    type= 'mask'  # rectangle polygon mask

    handle = vot.VOT(type)

    vot_anno = handle.region()
    print(str(vot_anno[0][0])+'-------------------')

    image_path = handle.frame()

    image_path = _convert_image_path(image_path)
    video_name = image_path.split('/')[-3]
    index = 0
    image = read_image(image_path)

    if type=='mask':
       vot_anno_mask = vot.make_full_size(vot_anno, (image.shape[1], image.shape[0]))
       bbox = masks_to_bboxes(torch.from_numpy(vot_anno_mask), fmt='t').squeeze().tolist()
    elif type=='rectangle':
       bbox = _convert_anno_to_list(vot_anno)
    elif type == 'polygon':
       bbox = [vot_anno[0][0][0], vot_anno[0][0][1], vot_anno[0][1][0], vot_anno[0][1][1],
                 vot_anno[0][2][0],vot_anno[0][2][1],vot_anno[0][3][0],vot_anno[0][3][1]]

    tracker_module_dimp_AMN = importlib.import_module('pytracking.tracker.{}'.format('dimp_AMN'))
    tracker_class_dimp_AMN = tracker_module_dimp_AMN.get_tracker_class()
    param_module_dimp_AMN = importlib.import_module(
        'pytracking.parameter.{}.{}'.format('dimp_AMN', 'dimp50_AMN_438'))  # dimp50_vot18
    params_dimp_AMN = param_module_dimp_AMN.parameters()
    dimp_AMN_tracker = tracker_class_dimp_AMN(params_dimp_AMN)

    tracker = dimp_AMN_tracker

    if type =='polygon':
        tracker.init_vot20(image, init_mask=None, init_bbox=bbox)
    elif type =='mask':
         tracker.init_vot20(image, init_mask=vot_anno_mask, init_bbox=bbox) #

    # Track
    while True:
        index = index+1
        image_path = handle.frame()
        if not image_path:
            break
        image_path = _convert_image_path(image_path)

        image = read_image(image_path)

        # info = OrderedDict()
        # info['previous_output'] = prev_output

        out = tracker.update_vot20(image)
        if type=='mask':
            pred = out['segmentation'].astype(np.uint8)
        elif type == 'polygon':
            pred_np =out['target_bbox']
            pred = vot.Polygon([vot.Point(pred_np[0], pred_np[1]),
                                     vot.Point(pred_np[2], pred_np[3]),
                                     vot.Point(pred_np[4], pred_np[5]),
                                     vot.Point(pred_np[6], pred_np[7])])
        #if video_name =='ants1':
        #    sns.heatmap(pred)
        #    plt.show()
        vis = False
        if vis:
            save_path = '/home/v-xfei/Public/dataset/vot_tools/vot20/vis/dimp_AMN/' + video_name
            if not os.path.exists(save_path):
                os.mkdir(save_path)

            image_ori = image[:, :, ::-1].copy()  # RGB --> BGR
            # cv2.imwrite(save_path, image_ori)

            if type == 'mask':
                # original image + mask
                image_m = image_ori.copy().astype(np.float32)
                image_m[:, :, 1] += 127.0 * pred
                image_m[:, :, 2] += 127.0 * pred
                contours, _ = cv2.findContours(pred, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)
                image_m = cv2.drawContours(image_m, contours, -1, (0, 255, 255), 2)
                image_m = image_m.clip(0, 255).astype(np.uint8)

                save_path = os.path.join(save_path, str(index) + '.jpg')
                cv2.imwrite(save_path, image_m)

            elif type == 'polygon':
                image_m = image_ori.copy().astype(np.float32)

                rot_box = np.array([[pred_np[0], pred_np[1]], [pred_np[2], pred_np[3]],
                                    [pred_np[4], pred_np[5]], [pred_np[6], pred_np[7]]], np.int32)
                cv2.polylines(image_m, [rot_box], isClosed=True, color=(255, 255, 0), thickness=2)

                image_m = image_m.clip(0, 255).astype(np.uint8)

                save_path = os.path.join(save_path, str(index) + '.jpg')
                cv2.imwrite(save_path, image_m)




        handle.report(pred, 1.0)



